from src.services.vessel_service import VesselService
from src.models.vessel import VesselResponse, VesselRequest
from src.models.generic import GenericObjectList
from src.models.enums import AscDescEnum, AggregateType, ResolutionType, EncodingType
from src.database.cache import Cache
from src.database.database_connector import DatabaseSingleton

from datetime import datetime, timedelta
from typing import Optional
from fastapi import APIRouter, Header, Response, status

router = APIRouter(
    prefix='/vessels',
    tags=['Vessels']
)

vessel_service = VesselService(db_context=DatabaseSingleton(),
                               cache_context=Cache())

responses = {
    200: {'description': 'Item found'},
    304: {'description': 'Not Modified'},
}


@router.post('/values',
             response_model=VesselResponse,
             responses={**responses})
async def get_data_points(
        response: Response,
        request_body: VesselRequest,
        measurement: str = 'testship',
        to_date: Optional[datetime] = datetime.now(),
        from_date: Optional[datetime] = (datetime.now() - timedelta(days=730)),
        order_by: str = 'time',
        asc_desc: AscDescEnum = AscDescEnum.ASCENDING,
        limit: Optional[int] = 100,
        offset: Optional[int] = None,
        encoding_type: EncodingType = EncodingType.DELTA,
        etag: Optional[str] = Header(None)):
    """
    Args:
        response: Response object sent back to client. Gets set automatically by FastAPI
        request_body: Request body from POST containing which fields to be fetched
        measurement: Name of measurement to be queried
        to_date: ISO date to fetch data to
        from_date: ISO date to fetch data from
        order_by: Which field to order by
        asc_desc: Order by descending order or ascending order
        limit: Max rows to be fetched per field
        offset: Static offset
        encoding_type: Time stamp encoding
        etag: Incoming Etag from client

    Returns: Response with data, or 304 not modified if Etag matches
    """
    args = locals()
    args.pop('response')  # Remove response object from hashing
    args.pop('etag')  # Remove etag from hashing

    etag_hash = str(vessel_service.hash_request_arguments(args))

    response.headers['etag'] = etag_hash

    if etag is not None and etag == etag_hash:
        response.status_code = status.HTTP_304_NOT_MODIFIED
        return response

    data = vessel_service.get_values(measurement=measurement,
                                     request_body=request_body,
                                     to_date=to_date,
                                     from_date=from_date,
                                     asc_desc=asc_desc,
                                     order_by=order_by,
                                     limit=limit,
                                     offset=offset,
                                     encoding_type=encoding_type)

    response.headers['cache-control'] = 'public'
    response_object = VesselResponse(result=data)
    return response_object


# TODO: Reformat response to contain fewer characters
@router.post('/values/aggregates',
             response_model=VesselResponse,
             responses={**responses})
async def get_data_point_aggregates(
        response: Response,
        request_body: VesselRequest,
        measurement: str = 'testship',
        aggregate_type: AggregateType = AggregateType.MEAN,
        resolution_type: ResolutionType = ResolutionType.MINUTE,
        resolution: int = 5,
        to_date: Optional[datetime] = datetime.now(),
        from_date: Optional[datetime] = (datetime.now() - timedelta(days=730)),
        limit: Optional[int] = 100,
        offset: Optional[int] = None,
        encoding_type: EncodingType = EncodingType.DELTA,
        etag: Optional[str] = Header(None)):
    """
    Args:
        response: Response object sent back to client. Gets set automatically by FastAPI
        request_body: Request body from POST containing which fields to be fetched
        measurement: Name of measurement to be queried
        aggregate_type: Which type of aggregation to apply
        resolution_type: Unit of time
        resolution: Time unit to aggregate by
        to_date: ISO date to fetch data from
        from_date: ISO date to fetch data from
        limit: Max rows to be fetched per field
        offset: Static offset
        encoding_type: Time stamp encoding
        etag: Incoming Etag from client

    Returns: Response with data, or 304 not modified if Etag matches
    """

    args = locals()
    args.pop('response')
    args.pop('etag')

    etag_hash = str(vessel_service.hash_request_arguments(args))
    response.headers['etag'] = etag_hash

    if etag is not None and etag == etag_hash:
        response.status_code = status.HTTP_304_NOT_MODIFIED
        return response

    data = vessel_service.get_multiple_aggregates(request_body=request_body,
                                                  measurement=measurement,
                                                  aggregate_type=aggregate_type,
                                                  resolution_type=resolution_type,
                                                  resolution=resolution,
                                                  to_date=to_date,
                                                  from_date=from_date,
                                                  limit=limit,
                                                  offset=offset,
                                                  encoding_type=encoding_type
                                                  )

    response.headers['cache-control'] = 'public'
    response_object = VesselResponse(result=data)
    return response_object


@router.get('/columns', response_model=GenericObjectList)
async def get_columns() -> GenericObjectList:
    """
    Returns: List of fields that are available
    """
    data = vessel_service.get_columns()
    response_object = GenericObjectList(values=data)
    return response_object
